<?php
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       http://myworks.design/
 * @since      1.0.0
 *
 * @package    MW_QBO_Desktop
 * @subpackage MW_QBO_Desktop/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    MW_QBO_Desktop
 * @subpackage MW_QBO_Desktop/admin
 * @author     MyWorks Design <sales@myworks.design>
 */
class MW_QBO_Desktop_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	 
	private $cur_db_version = '1.0.20';
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		
		global $MWQDC_LB,$MWQDC_AD;		
		$MWQDC_LB = new MW_QBO_Desktop_Sync_Lib();
		$MWQDC_AD = $this;
		
		require_once plugin_dir_path( __FILE__ ).'class-mw-qbo-desktop-ajax-actions.php';
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in MW_QBO_Desktop_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The MW_QBO_Desktop_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		
		$query_string = explode('=',$_SERVER['QUERY_STRING']);
		if(isset($query_string[1])){
			if( strpos( $query_string[1], "mw-qbo-desktop" ) !== false){
				wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/mw-qbo-desktop-admin.css', array(), $this->version, 'all' );
			}
			
			if( strpos( $query_string[1], "shop_order" ) !== false){
				wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/mw-qbo-desktop-admin-others.css', array(), $this->version, 'all' );
			}
		}

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in MW_QBO_Desktop_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The MW_QBO_Desktop_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		
		$query_string = explode('=',$_SERVER['QUERY_STRING']);
		if(isset($query_string[1])){
			if( strpos( $query_string[1], "mw-qbo-desktop" ) !== false || strpos( $query_string[1], "shop_order" ) !== false ) {
			    wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/mw-qbo-desktop-admin.js', array( 'jquery' ), $this->version, false );
			}			
		}

	}

	public function create_admin_menus() {
		global $MWQDC_LB;
		add_menu_page( 
			__( 'MyWorks Sync</br><span style="font-size:10px;">QuickBooks Desktop</span>', 'mw_wc_qbo_desk' ),
			__( 'MyWorks Sync</br><span style="font-size:10px;">QuickBooks Desktop</span>', 'mw_wc_qbo_desk' ), 
			'manage_options', 
			'mw-qbo-desktop', 
			array($this, 'mw_qbo_admin_menu'),
			plugin_dir_url( __FILE__ ) . 'image/menu-icon-sync.png', 
			3
		);

		add_submenu_page( 
			'mw-qbo-desktop', 
			__( 'Dashboard', 'mw_wc_qbo_desk' ),
			__( 'Dashboard', 'mw_wc_qbo_desk' ),
			'manage_options',
			'mw-qbo-desktop',
			array($this, 'mw_qbo_admin_menu')
		);
		
		if($MWQDC_LB->is_license_active()){
			$queue_count = $MWQDC_LB->get_queue_count();
			add_submenu_page( 
				'mw-qbo-desktop', 
				__( 'Queue', 'mw_wc_qbo_desk' ),
				__( 'Queue('.$queue_count.')', 'mw_wc_qbo_desk' ),
				'manage_options',
				'mw-qbo-desktop-queue',
				array($this, 'mw_qbo_admin_menu_queue')
			);
		}
		

		add_submenu_page( 
			'mw-qbo-desktop', 
			__( 'Connection', 'mw_wc_qbo_desk' ),
			__( 'Connection', 'mw_wc_qbo_desk' ),
			'manage_options',
			'mw-qbo-desktop-qwc-file',
			array($this, 'mw_qbo_admin_menu_qwc_file')
		);
		
		if($MWQDC_LB->is_license_active()){
			add_submenu_page( 
				'mw-qbo-desktop', 
				__( 'Settings', 'mw_wc_qbo_desk' ),
				__( 'Settings', 'mw_wc_qbo_desk' ),
				'manage_options',
				'mw-qbo-desktop-settings',
				array($this, 'mw_qbo_admin_menu_settings')
			);
			
			add_submenu_page( 
				'mw-qbo-desktop', 
				__( 'Log', 'mw_wc_qbo_desk' ),
				__( 'Log', 'mw_wc_qbo_desk' ),
				'manage_options',
				'mw-qbo-desktop-log',
				array($this, 'mw_qbo_admin_menu_log')
			);

			add_submenu_page( 
				'mw-qbo-desktop', 
				__( 'Map', 'mw_wc_qbo_desk' ),
				__( 'Map', 'mw_wc_qbo_desk' ),
				'manage_options',
				'mw-qbo-desktop-map',
				array($this, 'mw_qbo_admin_menu_map')
			);
			
			add_submenu_page( 
				'mw-qbo-desktop', 
				__( 'Push', 'mw_wc_qbo_desk' ),
				__( 'Push', 'mw_wc_qbo_desk' ),
				'manage_options',
				'mw-qbo-desktop-push',
				array($this, 'mw_qbo_admin_menu_push')
			);
			
			if($MWQDC_LB->option_checked('mw_wc_qbo_desk_pull_enable')){
				$sub_page = add_submenu_page( 
					'mw-qbo-desktop', 
					__( 'Pull', 'mw_wc_qbo_desk' ),
					__( 'Pull', 'mw_wc_qbo_desk' ),
					'manage_options',
					'mw-qbo-desktop-pull',
					array($this, 'mw_qbo_admin_menu_pull')
				);
			}
			
			add_submenu_page( 
				'mw-qbo-desktop', 
				__( 'Refresh Data', 'mw_wc_qbo_desk' ),
				__( 'Refresh Data', 'mw_wc_qbo_desk' ),
				'manage_options',
				'mw-qbo-desktop-refresh-data',
				array($this, 'mw_qbo_admin_menu_refresh_data')
			);

			add_submenu_page( 
				'mw-qbo-desktop', 
				__( 'Database Status', 'mw_wc_qbo_desk' ),
				__( 'Database Status', 'mw_wc_qbo_desk' ),
				'manage_options',
				'myworks-wc-qbd-sync-db-fix',
				array($this, 'myworks_wc_qbd_sync_db_fix')
			);
		}		

		/*add_submenu_page( 
			'mw-qbo-desktop', 
			__( 'Export', 'mw_wc_qbo_desk' ),
			__( 'Export - IIF', 'mw_wc_qbo_desk' ),
			'manage_options',
			'mw-qbo-desktop-export',
			array($this, 'mw_qbo_admin_menu_export')
		);*/
		
	}

	public function myworks_wc_qbd_sync_db_fix(){
		require_once plugin_dir_path( __FILE__ ) . 'partials/myworks-wc-qbd-sync-db-fix.php';
	}
	
	public function mw_qbo_admin_menu(){
		require_once plugin_dir_path( __FILE__ ) . 'partials/mw-qbo-desktop-admin-menu-dashboard.php';
	}

	public function mw_qbo_admin_menu_settings(){
		require_once plugin_dir_path( __FILE__ ) . 'partials/mw-qbo-desktop-admin-menu-settings.php';
	}
	
	public function mw_qbo_admin_menu_map(){
		require_once plugin_dir_path( __FILE__ ) . 'partials/mw-qbo-desktop-admin-menu-map.php';
	}
	
	public function mw_qbo_admin_menu_push(){
		require_once plugin_dir_path( __FILE__ ) . 'partials/mw-qbo-desktop-admin-menu-push.php';
	}
	
	public function mw_qbo_admin_menu_pull(){
		require_once plugin_dir_path( __FILE__ ) . 'partials/mw-qbo-desktop-admin-menu-pull.php';
	}	

	public function mw_qbo_admin_menu_export(){
		require_once plugin_dir_path( __FILE__ ) . 'partials/mw-qbo-desktop-admin-menu-export.php';
	}
	public function mw_qbo_admin_menu_qwc_file(){
		require_once plugin_dir_path( __FILE__ ) . 'partials/mw-qbo-desktop-admin-menu-qwc-file.php';
	}

	public function mw_qbo_admin_menu_queue(){
		require_once plugin_dir_path( __FILE__ ) . 'partials/mw-qbo-desktop-admin-menu-queue.php';
	}

	public function mw_qbo_admin_menu_log(){
		require_once plugin_dir_path( __FILE__ ) . 'partials/mw-qbo-desktop-admin-menu-log.php';
	}

	public function mw_qbo_admin_menu_refresh_data(){
		require_once plugin_dir_path( __FILE__ ) . 'partials/mw-qbo-desktop-admin-quick-refresh-data.php';
	}
	
	public function return_plugin_version(){
		$plugin_data = get_plugin_data( WP_PLUGIN_DIR . '/myworks-quickbooks-desktop-sync/myworks-quickbooks-desktop-sync.php', false, false );
		return $plugin_data['Version'];
	}

	public function fix_db_alter_issue($redirect=true,$rd_url=''){		
		global $MWQDC_LB;
		global $wpdb;
		$server_db = $MWQDC_LB->db_check_get_fields_details();
		
		if(is_array($server_db) && count($server_db)){
			foreach($server_db as $k=>$v){
				$is_db_updated = false;
				if($k == $wpdb->prefix.'mw_wc_qbo_desk_qbd_map_paymentmethod'){				
					if(!array_key_exists("term_id_str",$v)){					
						$sql = "ALTER TABLE `{$wpdb->prefix}mw_wc_qbo_desk_qbd_map_paymentmethod` CHANGE `term_id` `term_id_str` VARCHAR(255) CHARACTER SET latin1 COLLATE latin1_swedish_ci NOT NULL;";
						$wpdb->query($sql);
						$is_db_updated = true;					
					}
				}

				if($k == $wpdb->prefix.'mw_wc_qbo_desk_qbd_data_pairs'){
					if(!array_key_exists("ext_data",$v)){
						$sql = "ALTER TABLE `{$wpdb->prefix}mw_wc_qbo_desk_qbd_data_pairs` ADD `ext_data` VARCHAR(255) NOT NULL AFTER `d_type`;";
						$wpdb->query($sql);
						$is_db_updated = true;					
					}
				}
				
				if($is_db_updated && $redirect){
					if(empty($rd_url)){
						$rd_url = admin_url('admin.php?page=mw-qbo-desktop');
					}
					wp_redirect($rd_url);
					exit(0);
				}
			}
			
			//New Tables
			if(!isset($server_db[$wpdb->prefix.'mw_wc_qbo_desk_qbd_list_inventorysite'])){
				$sql = "CREATE TABLE IF NOT EXISTS {$wpdb->prefix}mw_wc_qbo_desk_qbd_list_inventorysite (
				  id int(11) NOT NULL AUTO_INCREMENT,
				  qbd_id varchar(255) NOT NULL,
				  name varchar(255) NOT NULL,
				  is_active int(1) NOT NULL,
				  is_default int(1) NOT NULL,
				  s_desc text NOT NULL,
				  `info_arr` longtext COLLATE utf8mb4_unicode_ci NOT NULL,
				  `info_qbxml_obj` longtext COLLATE utf8mb4_unicode_ci NOT NULL,
				  PRIMARY KEY (id)
				) ENGINE=MyISAM AUTO_INCREMENT=7 DEFAULT CHARSET=latin1;";
				$wpdb->query($sql);
			}
			
			/*
			if(!isset($server_db[$wpdb->prefix.'mw_wc_qbo_desk_qbd_list_othername'])){
				$sql = "CREATE TABLE IF NOT EXISTS {$wpdb->prefix}mw_wc_qbo_desk_qbd_list_othername (
				  id int(11) NOT NULL AUTO_INCREMENT,
				  qbd_id varchar(255) NOT NULL,
				  name varchar(255) NOT NULL,
				  companyname varchar(255) NOT NULL,
				  is_active int(1) NOT NULL,		  
				  `info_arr` longtext COLLATE utf8mb4_unicode_ci NOT NULL,
				  `info_qbxml_obj` longtext COLLATE utf8mb4_unicode_ci NOT NULL,
				  PRIMARY KEY (id)
				) ENGINE=MyISAM AUTO_INCREMENT=7 DEFAULT CHARSET=latin1;";
				$wpdb->query($sql);
			}
			*/
			
			if(!isset($server_db[$wpdb->prefix.'mw_wc_qbo_desk_qbd_list_salesrep'])){
				$sql = "CREATE TABLE IF NOT EXISTS {$wpdb->prefix}mw_wc_qbo_desk_qbd_list_salesrep (
				  id int(11) NOT NULL AUTO_INCREMENT,
				  qbd_id varchar(255) NOT NULL,				 
				  sr_e_ref_id varchar(255) NOT NULL,
				  sr_e_ref_name varchar(255) NOT NULL,
				  is_active int(1) NOT NULL,		  
				  `info_arr` longtext COLLATE utf8mb4_unicode_ci NOT NULL,
				  `info_qbxml_obj` longtext COLLATE utf8mb4_unicode_ci NOT NULL,
				  PRIMARY KEY (id)
				) ENGINE=MyISAM AUTO_INCREMENT=7 DEFAULT CHARSET=latin1;";
			}
			
			if(!isset($server_db[$wpdb->prefix.'mw_wc_qbo_desk_qbd_list_customertype'])){
				$sql = "CREATE TABLE IF NOT EXISTS {$wpdb->prefix}mw_wc_qbo_desk_qbd_list_customertype (
				  id int(11) NOT NULL AUTO_INCREMENT,
				  qbd_id varchar(255) NOT NULL,
				  name varchar(255) NOT NULL,		  
				  is_active int(1) NOT NULL,		  
				  `info_arr` longtext COLLATE utf8mb4_unicode_ci NOT NULL,
				  `info_qbxml_obj` longtext COLLATE utf8mb4_unicode_ci NOT NULL,
				  PRIMARY KEY (id)
				) ENGINE=MyISAM AUTO_INCREMENT=7 DEFAULT CHARSET=latin1;";
				$wpdb->query($sql);
			}
		}
		update_option('mw_wc_qbo_desk_cur_db_version',$this->cur_db_version);
	}
	
	public function mw_qbo_desktop_admin_init(){
		if(!session_id()) {
			session_start();
		}
		
		$this->mw_wc_qbo_enable_big_select_join();
		$this->mw_qbo_desktop_version_control();
		$this->mw_qbo_desktop_add_qbo_status_column();
		
		$this->mw_qbo_desktop_bulk_action_admin_footer();
	}
	
	public function mw_wc_qbo_enable_big_select_join(){
		global $wpdb;
		$wpdb->query('SET SESSION SQL_BIG_SELECTS = 1');
		$wpdb->query("SET SESSION sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''))");
	}
	
	public static function is_trial_version_check(){
		global $MWQDC_LB;
		if($MWQDC_LB->option_checked('mw_wc_qbo_desk_trial_license')){
			$query_string = explode('=',$_SERVER['QUERY_STRING']);
			if(isset($query_string[1])){
				if( strpos( $query_string[1], "mw-qbo-desktop" ) !== false ) {
					$image = plugin_dir_url( __FILE__ ) . 'image/Favicon-square.png';
				    echo '
					<div class="myworks-trial-container text-btn" style="position: relative;">
						<img width="25"  alt="MyWorks Desktop Sync" title="MyWorks Desktop Sync" src="'.$image.'">&nbsp;
						<h3><b>'.$MWQDC_LB->get_option('mw_wc_qbo_desk_trial_days_left').'</b> &nbsp;DAYS LEFT ON YOUR FREE TRIAL</h3>
						<a target="_blank" href="https://myworks.design/account/clientarea.php?action=productdetails&id='.$MWQDC_LB->get_option('mw_wc_qbo_desk_trial_license_serviceid').'" class="btn btn-info" role="button">UPGRADE NOW!
						</a>
						&nbsp;
						<a id="mwqs_tl_chk_again" style="font-size:12px; float: right; margin-left: -151px;padding: 29px; position: absolute; text-align: center;" href="javascript:void(0);">Check Again...</a>
					</div>
					';
					
					echo wp_nonce_field( 'myworks_wc_qbo_sync_trial_license_check_again_desk', 'trial_license_check_again_desk' );
					
					echo '
					<script>
					jQuery(document).ready(function($){
						$(\'#mwqs_tl_chk_again\').click(function(){
							jQuery(this).html(\'Loading...\');
							var data = {
								"action": \'mw_wc_qbo_sync_trial_license_check_again_desk\',
								"trial_license_check_again_desk": jQuery(\'#trial_license_check_again_desk\').val(),
							};
							jQuery.ajax({
							   type: "POST",
							   url: ajaxurl,
							   data: data,
							   cache:  false ,
							   //datatype: "json",
							   success: function(result){
								   if(result!=0 && result!=\'\'){
									location.reload();
								   }else{
									 jQuery(\'#mwqs_tl_chk_again\').html(\'Error!\');					 
								   }				  
							   },
							   error: function(result) { 		
									jQuery(\'#mwqs_tl_chk_again\').html(\'Error!\');
							   }
							});
						});
					});
					</script>
					';
							
				}
			}
		}
	}
	
	public function mw_qbo_desktop_bulk_action_admin_footer(){
		add_action('admin_footer-edit.php', 'custom_bulk_admin_footer');
		add_action('load-edit.php', 'custom_bulk_atq_process');
		
		function custom_bulk_admin_footer() {
			global $MWQDC_LB;
			global $post_type;
			if($post_type == 'shop_order') {
				$wco_ids = $MWQDC_LB->get_session_val('wc_order_id_list_ba_atq',array(),true);				
			?>
			<script type="text/javascript">
				jQuery(document).ready(function() {					
					jQuery('<option>').val('mw_qbd_add_to_queue').text('<?php _e('Add to Queue')?>').appendTo("select[name='action']");
					jQuery('<option>').val('mw_qbd_add_to_queue').text('<?php _e('Add to Queue')?>').appendTo("select[name='action2']");
					
					<?php 
						if(is_array($wco_ids) && count($wco_ids)):
						$sync_window_url = $MWQDC_LB->get_sync_window_url().'&sync_type=push&item_ids='.implode(',',$wco_ids).'&item_type=invoice';
					?>
					popUpWindowDesk('<?php echo $sync_window_url;?>','mw_qs_invoice_push_desk',0,0,650,350);
					<?php endif;?>
				});
			</script>
			<?php
			}
		}
		
		function custom_bulk_atq_process(){
			global $MWQDC_LB;
			global $typenow;
			$post_type = $typenow;
			
			if($post_type == 'shop_order') {
				$wp_list_table = _get_list_table('WP_Posts_List_Table');
				$action = $wp_list_table->current_action();
				$allowed_actions = array("mw_qbd_add_to_queue");
				if(!in_array($action, $allowed_actions)) return;
				
				check_admin_referer('bulk-posts');
				if(isset($_REQUEST['post'])) {
					$post_ids = array_map('intval', $_REQUEST['post']);
				}
				
				if(empty($post_ids)) return;
				$sendback = remove_query_arg( array('queued', 'untrashed', 'deleted', 'ids'), wp_get_referer() );
				if ( ! $sendback )
					$sendback = admin_url( "edit.php?post_type=$post_type" );
				
				$pagenum = $wp_list_table->get_pagenum();
				$sendback = add_query_arg( 'paged', $pagenum, $sendback );
				
				switch($action) {
					case 'mw_qbd_add_to_queue':
					$queued = 0;
					$MWQDC_LB->set_session_val('wc_order_id_list_ba_atq',$post_ids);
					//$sendback = add_query_arg( array('queued' => $queued, 'ids' => join(',', $post_ids) ), $sendback );
					break;
					
					default: return;
				}
				
				$sendback = remove_query_arg( array('action', 'action2', 'tags_input', 'post_author', 'comment_status', 'ping_status', '_status',  'post', 'bulk_edit', 'post_view'), $sendback );
				
				wp_redirect($sendback);
				exit();
			}
		}
	}
	
	public function mw_qbo_desktop_add_qbo_status_column(){
		add_filter( 'manage_edit-shop_order_columns', 'custom_shop_order_column',11);
		function custom_shop_order_column($columns){
		    $columns['mw_qbo_desktop_inv_status'] = __( 'QBD Status','mw_wc_qbo_desk');
		    return $columns;
		}
		
		add_action( 'manage_shop_order_posts_custom_column' , 'custom_orders_list_column_content_qbd', 10, 2 );
		function custom_orders_list_column_content_qbd( $column ){
			global $MWQDC_LB;			
		    global $post, $woocommerce, $the_order, $wpdb;
			
			/*
			$woo_version = $MWQDC_LB->get_woo_version_number();
			if ( $woo_version >= 3.0 ) {
				$order_id = (int) $the_order->get_id();
			}else{
				$order_id = (int) $the_order->id;
			}
			*/
			
			$order_id = 0;
			if(is_object($post) && count($post)){
				if(isset($post->ID) && isset($post->post_type) && $post->post_type == 'shop_order'){
					$order_id = (int) $post->ID;
				}
			}
			
			$sync_status_html = '<i class="fa fa-times-circle mwqd_wo_ic" style="color:red">Not Synced</i>';
			$dp_tbl = $wpdb->prefix.'mw_wc_qbo_desk_qbd_data_pairs';
			
			switch ( $column ){
		        case 'mw_qbo_desktop_inv_status' :
					if($order_id){
						$dp_data = $MWQDC_LB->get_row("SELECT `qbd_id` FROM {$dp_tbl} WHERE `wc_id` = {$order_id} AND `d_type` = 'Order' AND `qbd_id` !='' ");
						if(is_array($dp_data) && count($dp_data)){
							$sync_status_html = '<i title="QBD Txn ID #'.$dp_data['qbd_id'].'" class="fa fa-check-circle mwqd_wo_ic" style="color:green">Synced</i>';
						}else{
							$sync_window_url = $MWQDC_LB->get_sync_window_url().'&sync_type=push&item_ids='.$order_id.'&item_type=invoice';
							$sync_status_html.= '&nbsp;<a class="mwqd_wo_aqb" href="javascript:void(0)" onclick="javascript:popUpWindowDesk(\''.$sync_window_url.'\',\'mw_qs_invoice_push_desk\',0,0,650,350)">Add to queue</a>';
						}
					}
					echo $sync_status_html;
		            break;
		    }
		}
	}
	
	public function mw_qbo_desktop_version_control(){
		global $MWQDC_LB;
		//$mw_qbo_desktop_license = $MWQDC_LB->get_option('mw_qbo_desktop_license');
		//$mw_qbo_desktop_localkey = $MWQDC_LB->get_option('mw_qbo_desktop_localkey');
		//update_option('mw_qbd_sync_last_updated_version', $this->return_plugin_version());	
		//$mw_qbo_desktop_beta_update = $MWQDC_LB->get_option('mw_qbo_desktop_beta_update');
		
		/*
		if (version_compare(get_option('mw_qbd_sync_last_updated_version'), '1.0.15', '<=')){
			$this->fix_db_alter_issue();
		}
		*/		
		
		if($this->cur_db_version!=$MWQDC_LB->get_option('mw_wc_qbo_desk_cur_db_version')){
			$this->fix_db_alter_issue();
		}
		
		if( $MWQDC_LB->is_license_active() ){

			
			require_once( WP_PLUGIN_DIR . '/myworks-quickbooks-desktop-sync/wp-updates-plugin.php' );
			new WPUpdatesPluginUpdater_1707( 'http://wp-updates.com/api/2/plugin', 'myworks-quickbooks-desktop-sync/myworks-quickbooks-desktop-sync.php' );
			
			/*$version_control = new WPUpdatesPluginUpdater_1707( 'http://wp-updates.com/api/2/plugin', 'myworks-quickbooks-desktop-sync/myworks-quickbooks-desktop-sync.php' );
			echo '<h1>I AM HERE!</h1>';
			echo '<pre>';
            print_r($version_control);
            echo '</pre>';
			die;*/
			/*if($mw_qbo_desktop_beta_update == 'true'){
				require_once( WP_PLUGIN_DIR . '/myworks-quickbooks-desktop-sync/wp-updates-plugin-beta.php' );
				new WPUpdatesPluginUpdater_1604( 'http://wp-updates.com/api/2/plugin', 'myworks-quickbooks-desktop-sync/myworks-quickbooks-desktop-sync.php' );
			}*/
		}

		function mwqbd_db_pass_admin_notice() {

			echo '<div title="MyWorks QuickBooks Desktop Setup Error" class="notice notice-error mwqs-setup-notice">'.__('Illegal character detected. Please remove +, /, #, %, \, ? these from DB_NAME, DB_USER, DB_PASSWORD (if any)', 'mw_wc_qbo_desk').'</div>';
		}

		if($MWQDC_LB->get_session_val('unsupported_db_chars',false,true)){
			add_action( 'admin_notices', 'mwqbd_db_pass_admin_notice' );
		}
	}
	
	public function hook_variation_add($variation_info){
		if(!class_exists('WooCommerce')) return;
		global $MWQDC_LB;
		
		if(!$MWQDC_LB->is_qwc_connected()) {
			return false;
		}
		
		if(is_array($variation_info)){
			$variation_id = (int) $variation_info['variation_id'];
			$manual = true;
		}else{
			$variation_id = (int) $variation_info;
			$manual = false;
		}
		
		if(!$manual && !$MWQDC_LB->check_if_real_time_push_enable_for_item('variation')){
			return false;
		}
		
		if($variation_id>0){
			$variation = get_post($variation_id);
			if(!is_object($variation) || !count($variation)){
				if($manual){					
					$MWQDC_LB->save_log(array('log_type'=>'Variation','log_title'=>'Export Variation Error #'.$variation_id,'details'=>'Woocommerce variation not found','status'=>0));
				}
				return false;
			}
			
			if($variation->post_type!='product_variation'){
				if($manual){					
					$MWQDC_LB->save_log(array('log_type'=>'Variation','log_title'=>'Export Variation Error #'.$variation_id,'details'=>'Woocommerce variation is not valid.','status'=>0));
				}
				return false;
			}			
			
			if($variation->post_status=='auto-draft'){
				return false;
			}
			
			if(!$manual && $variation->post_status=='draft'){
				return false;
			}
			
			$variation_data = $MWQDC_LB->get_wc_variation_info($variation_id,$manual);
			
			if(!isset($variation_data['_manage_stock'])){
				return false;
			}
			
			$_manage_stock = $MWQDC_LB->get_array_isset($variation_data,'_manage_stock','no',true);
			$_downloadable = $MWQDC_LB->get_array_isset($variation_data,'_downloadable','no',true);
			$_virtual = $MWQDC_LB->get_array_isset($variation_data,'_virtual','no',true);				
			
			if($_manage_stock=='yes'){
				$qbd_item_type = 'V_'.QUICKBOOKS_ADD_INVENTORYITEM;
			}elseif($_virtual=='yes'){
				$qbd_item_type = 'V_'.QUICKBOOKS_ADD_SERVICEITEM;
			}else{
				$qbd_item_type = 'V_'.QUICKBOOKS_ADD_NONINVENTORYITEM;
			}
			
			if(!$quickbook_product_id = $MWQDC_LB->if_qbo_product_exists($variation_data)){	
				$priority = ($manual)?1:$variation_id;
				$Queue = new QuickBooks_WebConnector_Queue($MWQDC_LB->get_dsn());				
				$Queue->enqueue($qbd_item_type, $variation_id,$priority,array('is_variation'=>1),$MWQDC_LB->get_qbun());
				$MWQDC_LB->save_log(array('log_type'=>'Variation','log_title'=>'Export Variation #'.$variation_id,'details'=>'Variation added into queue','status'=>3));
				return true;
			}else{
				if($manual){					
					$MWQDC_LB->save_log(array('log_type'=>'Variation','log_title'=>'Export Variation Error #'.$variation_id,'details'=>'Variation already exists','status'=>0));				
				}
			}
			
		}
	}
	
	public function hook_product_add($product_info){
		if(!class_exists('WooCommerce')) return;
		global $MWQDC_LB;
		
		if(!$MWQDC_LB->is_qwc_connected()) {
			return false;
		}
		
		if(is_array($product_info)){
			$product_id = (int) $product_info['product_id'];
			$manual = true;
		}else{
			$product_id = (int) $product_info;
			$manual = false;
		}
		
		if(!$manual && !$MWQDC_LB->check_if_real_time_push_enable_for_item('product')){
			return false;
		}
		
		if($product_id>0){
			$_product = wc_get_product( $product_id );
			if(!count($_product)){
				$MWQDC_LB->save_log(array('log_type'=>'Product','log_title'=>'Export Product Error #'.$product_id,'details'=>'Woocommerce product not found','status'=>0));
				return false;
			}
			
			if($_product->post->post_status=='auto-draft'){
				return false;
			}
			
			if(!$manual && $_product->post->post_status=='draft'){
				return false;
			}
			
			$product_data = $MWQDC_LB->get_wc_product_info($product_id,$manual);
			
			if(!isset($product_data['_manage_stock'])){
				return false;
			}
			
			$_manage_stock = $MWQDC_LB->get_array_isset($product_data,'_manage_stock','no',true);
			$_downloadable = $MWQDC_LB->get_array_isset($product_data,'_downloadable','no',true);
			$_virtual = $MWQDC_LB->get_array_isset($product_data,'_virtual','no',true);				
			
			if($_manage_stock=='yes'){
				$qbd_item_type = QUICKBOOKS_ADD_INVENTORYITEM;
			}elseif($_virtual=='yes'){
				$qbd_item_type = QUICKBOOKS_ADD_SERVICEITEM;
			}else{
				$qbd_item_type = QUICKBOOKS_ADD_NONINVENTORYITEM;
			}
			
			if(!$quickbook_product_id = $MWQDC_LB->if_qbo_product_exists($product_data)){				
				$priority = ($manual)?1:$product_id;
				$Queue = new QuickBooks_WebConnector_Queue($MWQDC_LB->get_dsn());				
				$Queue->enqueue($qbd_item_type, $product_id,$priority,null,$MWQDC_LB->get_qbun());
				$MWQDC_LB->save_log(array('log_type'=>'Product','log_title'=>'Export Product #'.$product_id,'details'=>'Product added into queue','status'=>3));
				return true;
			}else{
				if($manual){					
					$MWQDC_LB->save_log(array('log_type'=>'Product','log_title'=>'Export Product Error #'.$product_id,'details'=>'Product already exists','status'=>0));				
				}
			}
		}
	}	
	
	public function hook_user_register($user_info,$from_order=false){
		if(!class_exists('WooCommerce')) return;
		global $MWQDC_LB;
		
		if(!$MWQDC_LB->is_qwc_connected()) {
			return false;
		}
		if(is_array($user_info)){
			$user_id = (int) $user_info['user_id'];
			$manual = true;
		}else{
			$user_id = (int) $user_info;
			$manual = false;
		}
		
		if(!$manual && !$from_order && !$MWQDC_LB->check_if_real_time_push_enable_for_item('customer')){	
			return false;
		}
		
		if(!$manual && $MWQDC_LB->is_plugin_active('woocommerce-aelia-currencyswitcher') && $MWQDC_LB->option_checked('mw_wc_qbo_desk_wacs_satoc_cb')){
			return false;
		}
		
		if($user_id){
			$user_data = get_userdata($user_id);
			
			if($MWQDC_LB->option_checked('mw_wc_qbo_desk_all_order_to_customer')){
				$io_cs = false;
				if(isset($user_data->roles) && is_array($user_data->roles)){
					$sc_roles_as_cus = $MWQDC_LB->get_option('mw_wc_qbo_desk_wc_cust_role_sync_as_cus');
					if(!empty($sc_roles_as_cus)){
						$sc_roles_as_cus = explode(',',$sc_roles_as_cus);
						if(is_array($sc_roles_as_cus) && count($sc_roles_as_cus)){
							foreach($sc_roles_as_cus as $sr){
								if(in_array($sr,$user_data->roles)){
									$io_cs = true;
									break;
								}
							}
						}
					}
				}
				if(!$io_cs){
					return false;
				}
			}
			
			$is_sync_user_role = false;
			if(isset($user_data->roles) && is_array($user_data->roles)){
				$sc_roles = $MWQDC_LB->get_option('mw_wc_qbo_desk_wc_cust_role');
				if(!empty($sc_roles)){
					$sc_roles = explode(',',$sc_roles);
					if(is_array($sc_roles) && count($sc_roles)){
						foreach($sc_roles as $sr){
							if(in_array($sr,$user_data->roles)){
								$is_sync_user_role = true;
								break;
							}
						}
					}
				}
				if(!$is_sync_user_role){
					if(in_array('customer',$user_data->roles)){
						$is_sync_user_role = true;						
					}
				}
			}			
			
			if($is_sync_user_role){
				$customer_data = $MWQDC_LB->get_wc_customer_info($user_id,$manual);
				
				if(!$qbd_customerid = $MWQDC_LB->if_qbo_customer_exists($customer_data)){					
					if($customer_data['firstname']!='' || $manual || $from_order){
						$Queue = new QuickBooks_WebConnector_Queue($MWQDC_LB->get_dsn());
						//$action, $ident = null, $priority = 0, $extra = null, $user = null, $qbxml = null, $replace = true
						$Queue->enqueue(QUICKBOOKS_ADD_CUSTOMER, $user_id,2,null,$MWQDC_LB->get_qbun());
						$MWQDC_LB->save_log(array('log_type'=>'Customer','log_title'=>'Export Customer #'.$customer_data['wc_customerid'],'details'=>'Customer added into queue','status'=>3));
						return true;
					}
					if($manual || $from_order){
						$MWQDC_LB->save_log(array('log_type'=>'Customer','log_title'=>'Export Customer Error #'.$customer_data['wc_customerid'],'details'=>'Customer name can\'t be empty','status'=>0));						
					}
					
				}else{
					if($manual){						
						$MWQDC_LB->save_log(array('log_type'=>'Customer','log_title'=>'Export Customer Error #'.$customer_data['wc_customerid'],'details'=>'Customer already exists','status'=>0));						
					}					
				}				
			}			
		}
		return false;
	}
	
	public function hook_order_add($order_info){		
		if(!class_exists('WooCommerce')) return;
		global $MWQDC_LB;
		
		if(!$MWQDC_LB->is_qwc_connected()) {
			return false;
		}
		if(is_array($order_info)){
			$order_id = (int) $order_info['order_id'];
			$manual = true;
		}else{
			$order_id = (int) $order_info;
			$manual = false;
		}
		
		if(!$manual && !$MWQDC_LB->check_if_real_time_push_enable_for_item('order')){		
			return false;
		}
		
		if($order_id){
			$order = get_post($order_id);
			
			if(!is_object($order) || !count($order)){				
				if($manual){					
					$MWQDC_LB->save_log(array('log_type'=>'Invoice','log_title'=>'Export Order Error #'.$order_id,'details'=>'Woocommerce order not found.','status'=>0));
				}
				return false;
			}
			if($order->post_type!='shop_order'){
				if($manual){
					$MWQDC_LB->save_log(array('log_type'=>'Invoice','log_title'=>'Export Order Error #'.$order_id,'details'=>'Woocommerce order is not valid.','status'=>0));
				}
				return false;
			}
			
			if($order->post_status=='auto-draft'){
				return false;
			}
			
			if(!$manual && $order->post_status=='draft'){
				return false;
			}
			
			$invoice_data = $MWQDC_LB->get_wc_order_details_from_order($order_id,$order);
			/*
			$wc_inv_num = $MWQDC_LB->get_array_isset($invoice_data,'wc_inv_num','');
			$ord_id_num = ($wc_inv_num!='')?$wc_inv_num:$order_id;
			*/
			
			$is_os_p_sync = false;$is_os_err = false;
			
			
			$only_sync_status = $MWQDC_LB->get_option('mw_wc_qbo_desk_specific_order_status');
			if($only_sync_status!=''){$only_sync_status = explode(',',$only_sync_status);}
			
			if(!$manual && (!is_array($only_sync_status) || (is_array($only_sync_status) && !in_array($order->post_status,$only_sync_status)))){			
				//$is_os_err = true;
				return false;
			}			
			
			if(!is_array($invoice_data) || !isset($invoice_data['_order_key']) || !isset($invoice_data['_order_total'])){
				if($manual && !$is_os_err){					
					$MWQDC_LB->save_log(array('log_type'=>'Invoice','log_title'=>'Export Order Error #'.$order_id,'details'=>'Woocommerce order details not found.','status'=>0));
				}
				/*
				if($is_os_p_sync){			
					$MWQDC_LB->save_log(array('log_type'=>'Payment','log_title'=>'Export Payment Error Order #'.$order_id,'details'=>'Woocommerce order details not found.','status'=>0));
				}
				*/
				return false;
			}
			
			//Prevent Multiple Entry
			/*
			if(!$manual && !is_admin()){
				if((int) $MWQDC_LB->get_session_val('current_rt_order_id_'.$order_id,0)==$order_id){
					return false;
				}
				$MWQDC_LB->set_session_val('current_rt_order_id_'.$order_id,$order_id);
			}
			*/
			
			if(!$MWQDC_LB->check_quickbooks_invoice($order_id)){
				$is_valid = true;
				/*Customer Check*/
				
				$qbo_cus_id = '';
				if($MWQDC_LB->is_plugin_active('woocommerce-aelia-currencyswitcher') && $MWQDC_LB->option_checked('mw_wc_qbo_desk_wacs_satoc_cb')){
					$_order_currency = $MWQDC_LB->get_array_isset($invoice_data,'_order_currency','',true);
					if($_order_currency!=''){
						$aelia_cur_cus_map = get_option('mw_wc_qbo_desk_wacs_satoc_map_cur_cus');
						if(is_array($aelia_cur_cus_map) && count($aelia_cur_cus_map)){
							if(isset($aelia_cur_cus_map[$_order_currency]) && trim($aelia_cur_cus_map[$_order_currency])!=''){
								$qbo_cus_id = trim($aelia_cur_cus_map[$_order_currency]);
							}
						}
					}					
				}
				
				if(empty($qbo_cus_id)){
					if(!$MWQDC_LB->option_checked('mw_wc_qbo_desk_all_order_to_customer')){
						$wc_cus_id = (int) get_post_meta($order_id,'_customer_user',true);
						if($wc_cus_id>0){
							$customer_data = $MWQDC_LB->get_wc_customer_info($wc_cus_id);
							$qbo_cus_id = $MWQDC_LB->if_qbo_customer_exists($customer_data);
							if(empty($qbo_cus_id)){
								$this->hook_user_register($wc_cus_id,true);
							}					
						}else{
							$customer_data = $MWQDC_LB->get_wc_customer_info_from_order($order_id);
							$qbo_cus_id = $MWQDC_LB->if_qbo_guest_exists($customer_data);
							if(empty($qbo_cus_id)){
								$Queue = new QuickBooks_WebConnector_Queue($MWQDC_LB->get_dsn());
								$Queue->enqueue(QUICKBOOKS_ADD_GUEST, $order_id,2,null,$MWQDC_LB->get_qbun());
								$MWQDC_LB->save_log(array('log_type'=>'Customer','log_title'=>'Export Guest/Customer','details'=>'Customer added into queue','status'=>3));
							}
						}
					}else{
						//New Option
						if($wc_cus_id>0){
							$user_info = get_userdata($wc_cus_id);
							$io_cs = false;
							if(isset($user_info->roles) && is_array($user_info->roles)){
								$sc_roles_as_cus = $MWQDC_LB->get_option('mw_wc_qbo_desk_wc_cust_role_sync_as_cus');
								if(!empty($sc_roles_as_cus)){
									$sc_roles_as_cus = explode(',',$sc_roles_as_cus);
									if(is_array($sc_roles_as_cus) && count($sc_roles_as_cus)){
										foreach($sc_roles_as_cus as $sr){
											if(in_array($sr,$user_info->roles)){
												$io_cs = true;
												break;
											}
										}
									}
								}
							}
							
							if($io_cs){
								$customer_data = $MWQDC_LB->get_wc_customer_info($wc_cus_id);
								$qbo_cus_id = $MWQDC_LB->if_qbo_customer_exists($customer_data);
								if(empty($qbo_cus_id)){
									$this->hook_user_register($wc_cus_id,true);
								}
							}
						}
					}
				}							
				
				if($is_valid){
					$Queue = new QuickBooks_WebConnector_Queue($MWQDC_LB->get_dsn());
					if($MWQDC_LB->option_checked('mw_wc_qbo_desk_order_as_sales_receipt')){
						$Queue->enqueue(QUICKBOOKS_ADD_SALESRECEIPT, $order_id,1,null,$MWQDC_LB->get_qbun());
						
					}elseif($MWQDC_LB->option_checked('mw_wc_qbo_desk_order_as_sales_order')){
						$Queue->enqueue(QUICKBOOKS_ADD_SALESORDER, $order_id,1,null,$MWQDC_LB->get_qbun());
						
					}else{
						$Queue->enqueue(QUICKBOOKS_ADD_INVOICE, $order_id,1,null,$MWQDC_LB->get_qbun());
					}
					
					$MWQDC_LB->save_log(array('log_type'=>'Order','log_title'=>'Export Order #'.$order_id,'details'=>'Order added into queue','status'=>3));
					return true;
				}
				
			}else{
				if($manual){				
					$MWQDC_LB->save_log(array('log_type'=>'Order','log_title'=>'Export Order Error #'.$order_id,'details'=>'Order already exists','status'=>0));						
				}					
			}
			
		}
		return false;
	}
	
	public function hook_payment_add($pmnt_sync_info=''){
		if(!class_exists('WooCommerce')) return;
		global $MWQDC_LB;
		if(!$MWQDC_LB->is_qwc_connected()) {
			return false;
		}
		
		global $wpdb;
		$_transaction_id = '';
		$payment_id = 0;
		if(is_array($pmnt_sync_info)){
			$payment_id = (int) $pmnt_sync_info['payment_id'];
			$_transaction_id = $MWQDC_LB->get_field_by_val($wpdb->postmeta,'meta_value','meta_id',$payment_id);
			$order_id = (int) $MWQDC_LB->get_field_by_val($wpdb->postmeta,'post_id','meta_id',$payment_id);
			$manual = true;
		}else{
			$order_id = (int) $pmnt_sync_info;
			$_transaction_id = get_post_meta($order_id, '_transaction_id', true );
			$manual = false;
		}
		
		if($manual && !$payment_id){
			return false;
		}
		
		if(!$manual && !$MWQDC_LB->check_if_real_time_push_enable_for_item('payment')){
			return false;
		}
		
		if($MWQDC_LB->option_checked('mw_wc_qbo_desk_order_as_sales_receipt')){
			return false;
		}
		
		if($MWQDC_LB->option_checked('mw_wc_qbo_desk_order_as_sales_order')){
			return false;
		}
		
		$wc_inv_no = '';
		
		if($MWQDC_LB->is_plugin_active('woocommerce-sequential-order-numbers-pro','woocommerce-sequential-order-numbers') && $MWQDC_LB->option_checked('mw_wc_qbo_sync_compt_p_wsnop')){
			if($MWQDC_LB->is_plugin_active('woocommerce-sequential-order-numbers')){
				$wc_inv_no = get_post_meta( $order_id, '_order_number', true );
			}else{
				$wc_inv_no = get_post_meta( $order_id, '_order_number_formatted', true );
			}			
		}		
		
		$ord_id_num = ($wc_inv_no!='')?$wc_inv_no:$order_id;
		
		$_transaction_id_tmp = (!$_transaction_id)?'TXN-'.$order_id:$_transaction_id;
		if($_transaction_id_tmp){
			$payment_data = $MWQDC_LB->wc_get_payment_details_by_txn_id($_transaction_id,$order_id);
			if(!count($payment_data)){				
				if($manual){
					$MWQDC_LB->save_log(array('log_type'=>'Payment','log_title'=>'Export Payment Error #'.$payment_id,'details'=>'Woocommerce payment info not found','status'=>0));
				}else{			
					$MWQDC_LB->save_log(array('log_type'=>'Payment','log_title'=>'Export Payment Error for Order #'.$ord_id_num,'details'=>'Woocommerce payment info not found','status'=>0));				
				}
				return false;
			}
			
			//10-11-2017
			$payment_id = (int) $MWQDC_LB->get_array_isset($payment_data,'payment_id','',true);
			if($payment_id<1){
				$MWQDC_LB->save_log(array('log_type'=>'Payment','log_title'=>'Export Payment Error for Order #'.$ord_id_num,'details'=>'Woocommerce payment ID not found','status'=>0));
				return false;
			}
			
			$_payment_method = $MWQDC_LB->get_array_isset($payment_data,'payment_method','',true);
			$_payment_method_title = $MWQDC_LB->get_array_isset($payment_data,'payment_method_title','',true);
			
			$_order_currency = $MWQDC_LB->get_array_isset($payment_data,'order_currency','',true);
			
			$base_currency = '';
			if($MWQDC_LB->wacs_base_cur_enabled()){
				$base_currency = get_woocommerce_currency();
				$pm_map_data = $MWQDC_LB->get_mapped_payment_method_data($_payment_method,$base_currency);
			}else{
				$pm_map_data = $MWQDC_LB->get_mapped_payment_method_data($_payment_method,$_order_currency);
			}							
			
			$enable_payment = (int) $MWQDC_LB->get_array_isset($pm_map_data,'enable_payment',0);
			if($MWQDC_LB->wacs_base_cur_enabled()){
				$payment_amount = get_post_meta($order_id,'_order_total_base_currency',true);
			}else{
				$payment_amount = $MWQDC_LB->get_array_isset($payment_data,'order_total',0);
			}
			
			$payment_amount = floatval($payment_amount);
			
			$is_valid_payment = false;
			if($enable_payment && $payment_amount>0){
				$is_valid_payment = true;
			}
			$log_currency = ($base_currency!='')?$base_currency:$_order_currency;
			
			if(!$is_valid_payment){
				if($manual){
					$MWQDC_LB->save_log(array('log_type'=>'Payment','log_title'=>'Export Payment Error #'.$payment_id,'details'=>'Payment not enabled or invalid payment amount for gateway:'.$_payment_method.', currency:'.$log_currency,'status'=>0));
				}
				return false;
			}
			
			if(!$MWQDC_LB->check_quickbooks_payment($payment_id)){
				$Queue = new QuickBooks_WebConnector_Queue($MWQDC_LB->get_dsn());
				$Queue->enqueue('ReceivePaymentAdd', $payment_id,0,null,$MWQDC_LB->get_qbun());
				$MWQDC_LB->save_log(array('log_type'=>'Payment','log_title'=>'Export Payment #'.$payment_id,'details'=>'Payment added into queue','status'=>3));
				return true;
			}else{
				if($manual){
					$MWQDC_LB->save_log(array('log_type'=>'Payment','log_title'=>'Export Payment Error #'.$payment_data['payment_id'],'details'=>'Payment already exists','status'=>0));
				}
			}
		}else{			
			$MWQDC_LB->save_log(array('log_type'=>'Payment','log_title'=>'Export Payment Error for Order #'.$ord_id_num,'details'=>'Woocommerce Payment TXN ID Not Found','status'=>0));
		}
		
		return false;
	}
	
	public function hook_product_stock_update($ivnt_sync_info){
		if(!class_exists('WooCommerce')) return;
		global $MWQDC_LB;
		if(!$MWQDC_LB->is_qwc_connected()) {
			return false;
		}		
		$product_id = 0;		
		if(is_array($ivnt_sync_info)){
			$product_id = (int) $ivnt_sync_info['product_id'];
			$manual = true;
		}else{
			if(is_object($ivnt_sync_info) && count($ivnt_sync_info)){
				$product_id = (int) $ivnt_sync_info->get_id();
			}
			$manual = false;
		}		
		
		if(!$manual && !$MWQDC_LB->check_if_real_time_push_enable_for_item('inventory')){
			return false;
		}
		if($product_id){
			$_product = wc_get_product( $product_id );
			if(!count($_product)){
				$MWQDC_LB->save_log(array('log_type'=>'Inventory','log_title'=>'Export Inventory Error #'.$product_id,'details'=>'Woocommerce product not found','status'=>0));
				return false;
			}
			
			$product_data = $MWQDC_LB->get_wc_product_info($product_id,$manual);
			$_manage_stock = $MWQDC_LB->get_array_isset($product_data,'_manage_stock','no',true);
			
			if($_manage_stock!='yes'){
				//$MWQDC_LB->save_log(array('log_type'=>'Inventory','log_title'=>'Export Inventory Error #'.$product_id,'details'=>'Invalid Woocommerce inventory','status'=>0));
				return false;
			}
			
			if(!$quickbook_product_id = $MWQDC_LB->if_qbo_product_exists($product_data,true)){
				if($manual){
					$MWQDC_LB->save_log(array('log_type'=>'Inventory','log_title'=>'Export Inventory Error #'.$product_id,'details'=>'QuickBooks inventory not found','status'=>0));
				}
				return false;
			}
			
			//$_stock = $MWQDC_LB->get_array_isset($product_data,'_stock',0,true);
			
			//$priority = ($manual)?1:$product_id;
			$priority = 4;
			$Queue = new QuickBooks_WebConnector_Queue($MWQDC_LB->get_dsn());		
			$Queue->enqueue('InventoryAdjustmentAdd', $product_id,$priority,null,$MWQDC_LB->get_qbun());
			$MWQDC_LB->save_log(array('log_type'=>'Inventory','log_title'=>'Export Inventory #'.$product_id,'details'=>'Inventory added into queue','status'=>3));
			return true;
			
			
		}
		return false;
		
	}
	
	public function hook_variation_stock_update($ivnt_sync_info){
		if(!class_exists('WooCommerce')) return;
		global $MWQDC_LB;
		if(!$MWQDC_LB->is_qwc_connected()) {
			return false;
		}		
		$variation_id = 0;		
		if(is_array($ivnt_sync_info)){
			$variation_id = (int) $ivnt_sync_info['variation_id'];
			$manual = true;
		}else{
			if(is_object($ivnt_sync_info) && count($ivnt_sync_info)){
				$variation_id = (int) $ivnt_sync_info->get_id();
			}
			$manual = false;
		}		
		
		if(!$manual && !$MWQDC_LB->check_if_real_time_push_enable_for_item('inventory')){
			return false;
		}
		if($variation_id){
			$variation = get_post($variation_id);
			if(!count($variation)){
				$MWQDC_LB->save_log(array('log_type'=>'Inventory','log_title'=>'Export Variation Inventory Error #'.$variation_id,'details'=>'Woocommerce product not found','status'=>0));
				return false;
			}
			
			$variation_data = $MWQDC_LB->get_wc_variation_info($variation_id,$manual);
			$_manage_stock = $MWQDC_LB->get_array_isset($variation_data,'_manage_stock','no',true);
			
			if($_manage_stock!='yes'){
				//$MWQDC_LB->save_log(array('log_type'=>'Inventory','log_title'=>'Export Variation Inventory Error #'.$variation_id,'details'=>'Invalid Woocommerce inventory','status'=>0));
				return false;
			}
			
			if(!$quickbook_product_id = $MWQDC_LB->if_qbo_product_exists($variation_data,true)){
				if($manual){
					$MWQDC_LB->save_log(array('log_type'=>'Inventory','log_title'=>'Export Variation Inventory Error #'.$variation_id,'details'=>'QuickBooks inventory not found','status'=>0));
				}
				return false;
			}
			
			//$_stock = $MWQDC_LB->get_array_isset($variation_data,'_stock',0,true);
			
			//$priority = ($manual)?1:$variation_id;
			$priority = 4;
			$Queue = new QuickBooks_WebConnector_Queue($MWQDC_LB->get_dsn());	
			$Queue->enqueue('V_InventoryAdjustmentAdd', $variation_id,$priority,array('is_variation'=>1),$MWQDC_LB->get_qbun());
			$MWQDC_LB->save_log(array('log_type'=>'Inventory','log_title'=>'Export Variation Inventory #'.$variation_id,'details'=>'Inventory added into queue','status'=>3));
			return true;
			
			
		}
		return false;
		
	}
	
	public static function return_qbd_plugin_version(){
		$plugin_data = get_plugin_data( WP_PLUGIN_DIR . '/myworks-quickbooks-desktop-sync/myworks-quickbooks-desktop-sync.php', false, false );
		return $plugin_data['Version'];
	}

}
